require("tools")

local opFile = c8y.config:value("opFile", "/var/lib/sragent/configFile.op")
local inProgress = false
local tempFile = c8y.config:value("tempFile", "/tmp/configFile.cfg.tar.gz")
local templates = {}

local debugEnable = c8y.config:boolValue("debug") or false
local debug

if debugEnable then
  debug = print
else
  debug = function() end
end


local function download(url)
  local connection = c8y.net:get(url)
  connection:setOutFile(tempFile)
  connection.timeout = 10
  connection:send()
  connection:wait()
  local success, errorText
  if not connection.isFinished then
    errorText = "Connection timeout"
  elseif connection.isAborted then
    errorText = "Connection aborted"
  elseif not file_exists(tempFile) then
    errorText = "Creating local file failed"
  elseif connection.ok then
    success = true
  end
  connection:close()
  return success, errorText
end

local function saveOperation(id)
  local file = io.open(opFile, "w+")
  file:write(id)
  file:close()
end

local function installFile()
  debug("INSTALL FILE")
  local file = io.popen("install_file " .. tempFile)
  local response = file:read("*a")
  if response:find("is successful") then
    return true
  else
    return false
  end
end

local function reboot()
  debug("REBOOT")
  os.execute("reboot")
end
-----------------------------------------------------------------------
class("DownloadHandler")

function DownloadHandler:operationExecuting(op)
  if inProgress then
    op:setStatus(op.Failed, "Config operation already in progress")
  else
    inProgress = true
    local url = op:getResponseRecord(1)[3]
    debug("EXECUTING DOWNLOAD HANDLER", op.id, url)
    local success, errorText = download(url)
    if success then
      saveOperation(op.id)
      if installFile() then
        reboot()
      else
        op:setStatus(op.Failed, "Installing the config file failed")
      end
    else
      op:setStatus(op.Failed, errorText or "Unknown error")
    end
    inProgress = false
  end
end
-----------------------------------------------------------------------

local function exportConfig()
  local fileName = "NTC-6200_Settings_" .. c8y.device.id .. ".cfg"
  if os.execute("dbcfg_export -o /tmp/" .. fileName ..  ' -p ""') ~= 0 then
  	return nil, "Exporting rdb config failed"
  end
  if  os.execute('tar -C /usr/local/cdcs -zcvf - ipsec.d openvpn-keys ssh-hostkeys | openssl des3 -salt -k "" | dd of=/tmp/vpn.des3') ~= 0 then
  	return nil, "Exporting vpn config failed"
  end
  if os.execute("cd /tmp/ && tar -zcvf /opt/cdcs/upload/" .. fileName .. ".tar.gz " .. fileName .. " vpn.des3") ~= 0 then
  	return nil, "Archiving config failed"
  end 
  return fileName ..  ".tar.gz"
end

local function getHost()
  local host = c8y.config.main:value("connection.server")
  return host:match("https?:\/\/.*\/")
end

local function upload(fileName)
  local connection = c8y.net:post(getHost() .. "inventory/binaries")
  local file = io.open("/opt/cdcs/upload/" .. fileName)
  connection.timeout = 10
  connection:setHeader("Accept: application/vnd.com.nsn.cumulocity.managedObject+json;")
  connection:setHeader("Content-Type: multipart/form-data; boundary=----WebKitFormBoundary7MA4YWxkTrZu0gW")
  connection:write("------WebKitFormBoundary7MA4YWxkTrZu0gW\r\n")
  connection:write('Content-Disposition: form-data; name="object"\r\n\r\n')
  connection:write('{"name":"' .. fileName .. '","type":"application/gzip"}\r\n')
  connection:write("------WebKitFormBoundary7MA4YWxkTrZu0gW\r\n")
  connection:write('Content-Disposition: form-data; name="filesize"\r\n\r\n')
  connection:write(file:seek("end") .. "\r\n")
  connection:write("------WebKitFormBoundary7MA4YWxkTrZu0gW\r\n")
  connection:write('Content-Disposition: form-data; name="file"; filename="' .. fileName .. '"\r\n')
  connection:write("Content-Type: application/gzip\r\n\r\n")
  file:seek("set")
  connection:write(file:read("*a"))
  connection:write("\r\n------WebKitFormBoundary7MA4YWxkTrZu0gW--\r\n")
  connection:send()
  connection:wait()
  local binaryId, errorText
  debug("UPLOAD DONE", connection.ok, connection.statusCode)
  local response = connection:read()
  debug("RESPONSE", response)
  if connection.ok then
    binaryId = response:match('"id"%s*:%s*"(%d+)"')
    debug("BINARY ID", binaryId)
  else
    errorText = "Uploading config file failed: http code " .. (connection.statusCode or -1)
  end
  connection:close()
  return binaryId, errorText
end
-----------------------------------------------------------------------
class("UploadHandler")

function UploadHandler:operationExecuting(op)
  debug("EXECUTING UPLOAD HANDLER", op.id)
  local fileName, errorText = exportConfig()
  if not fileName then
    op:setStatus(op.Failed, errorText)
    return
  end
  local binaryId, errorText = upload(fileName)
  if not binaryId then
  	op:setStatus(op.Failed, errorText)
  	return
  end
  templates.updateOperation:send({op.id, binaryId})
  c8y.operations:opeartionHandled(op.id)
end
-----------------------------------------------------------------------

function init()
  local downloadHandler = DownloadHandler()
  c8y.operations:addOperation("c8y_DownloadConfigFile","$.url", downloadHandler)
  local uploadHandler = UploadHandler()
  c8y.operations:addOperation("c8y_UploadConfigFile","", uploadHandler)
  templates.updateOperation = c8y.templates:addRequest('PUT,/devicecontrol/operations/%%,application/vnd.com.nsn.cumulocity.operation+json,,%%,UNSIGNED UNSIGNED,"{""status"":""SUCCESSFUL"",""binaryId"":""%%""}"')
end

function start()
  local file = io.open(opFile)
  if file then
    debug("OP FILE EXISTS")
    local id = file:read("*n")
    debug("ID FROM OP FILE", id)
    if id then
      c8y.operations:setOperationStatus(id, c8y.operations.Successful)
    end
    file:close()
    os.remove(opFile)
  end
end
